#
# sercon.py - serial low level connection
#
# Written by
#  Christian Vogelgsang <chris@vogelgsang.org>
#
# This file is part of dtv2ser.
# See README for copyright notice.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
#  02111-1307  USA.
#

# requires PySerial from http://pyserial.sourceforge.net !

import serial
import time

from dtv2ser.status import *

class SerCon:
  """Encapsulates a low level serial connection to a dtv2ser server device."""

  # time out to wait for server become ready
  ready_timeout = 50
  
  def __init__(self, serial_port, serial_baud, serial_timeout=2):
    """Open a serial connection to the dtv2ser server."""
    try:
      self.ser = serial.Serial(port=serial_port,
                              baudrate=serial_baud,
                              bytesize=serial.EIGHTBITS,
                              parity=serial.PARITY_NONE,
                              stopbits=serial.STOPBITS_ONE,
                              timeout=serial_timeout,
                              writeTimeout=serial_timeout,
                              xonxoff=0,
                              rtscts=1)
      self.ser.open()
      if self.ser.isOpen():
        self.ser.flushInput()
        self.ser.flushOutput()
        self.valid = True
      else:
        self.valid = False
    except:
      self.valid = False
      
  def __del__(self):
    """Close connection to dtv2 server."""
    if self.valid and self.ser.isOpen():
      self.ser.close()
      
  def is_connected(self):
    """Is client connected?
    Returns True if the server is ready, otherwise False.
    """
    return self.valid and self.ser.isOpen()
    
  def wait_for_server(self,timeout=0):
    """Wait for server to become ready.
    Returns result code.
    """
    if timeout == 0:
      timeout = self.ready_timeout
    for t in xrange(timeout):
      if self.ser.getCTS():
        return STATUS_OK
      time.sleep(0.1)
    return CLIENT_ERROR_SERVER_TIMEOUT

  # ----- basic I/O -----
  
  def receive_data(self,length):
    """Read n bytes from server.
    Returns (result,data).
    """
    result = self.ser.read(length)
    if len(result) != length:
      return (CLIENT_ERROR_SERVER_TIMEOUT,0)
    return (STATUS_OK,result)

  def send_data(self,data):
    """Write n bytes to server.
    Returns result.
    """
    try:
      self.ser.write(data)
    except(writeTimeOutError):
      return CLIENT_ERROR_SERVER_TIMEOUT
    return STATUS_OK

  def receive_block(self,length,block_size,callback=lambda x:True):
    """Send a large block of data in blocks given by block_size.
    Use optional callback to get feedback while transfer.
    Returns (result,duration,data).
    """
    # how many blocks
    blocks    = length / block_size
    remainder = length % block_size

    # begin download
    start_time = time.time()
  
    # transfer blocks
    pos = 0
    data = ''
    error = 0
    for b in xrange(blocks):
      d = self.ser.read(block_size)
      l = len(d)
      if l < block_size:
        error = 1
        break
      data += d
      pos += block_size
      callback(pos)

    # transfer remainder
    if remainder != 0 and error == 0:
      d = self.ser.read(remainder)
      l = len(d)
      if l < remainder:
        error = 1
      else:
        data += d
      callback(pos)
    
    if error:
      return (CLIENT_ERROR_SERVER_TIMEOUT,pos,data) 
    
    # end download
    end_time = time.time()
    
    # all ok
    return (STATUS_OK,end_time - start_time,data)

  def send_block(self,data,block_size,callback=lambda x:True):
    """Send a large block of data in blocks given by block_size.
    Use optional callback to get feedback while transfer.
    Returns (result,duration).
    """
    # how many blocks
    length    = len(data)
    blocks    = length / block_size
    remainder = length % block_size
    
    # begin upload
    start_time = time.time()

    # transfer blocks
    try:
      pos = 0
      for b in xrange(blocks):
        d = data[pos:pos+block_size]
        l = len(d)
        self.ser.write(d)
        pos += block_size
        callback(pos)

      # transfer remainder
      if remainder != 0:
        d = data[pos:]
        self.ser.write(d)
        callback(length)
    
    # time out?
    except(writeTimeOutError):
      return (CLIENT_ERROR_SERVER_TIMEOUT,pos)

    # end upload
    end_time = time.time()
    
    # all ok
    return (STATUS_OK,end_time - start_time)
